import { SmoothScrollElement, SmoothScrollToElementOptions, SmoothScrollToOptions } from './smooth-scroll.model';
import * as i0 from "@angular/core";
export declare class SmoothScrollManager {
    private _document;
    private _platform;
    private readonly rtlScrollAxisType;
    private readonly _defaultOptions;
    private _onGoingScrolls;
    private get _w();
    /**
     * Timing method
     */
    private get _now();
    constructor(_document: Document, _platform: object, customDefaultOptions: SmoothScrollToOptions);
    /**
     * changes scroll position inside an element
     */
    private _scrollElement;
    /**
     * Handles a given parameter of type HTMLElement, ElementRef or selector
     */
    private _getElement;
    /**
     * Initializes a destroyer stream, re-initializes it if the element is already being scrolled
     */
    private _initSmoothScroll;
    /**
     * Checks if smooth scroll has reached, cleans up the smooth scroll stream and resolves its promise
     */
    private _isFinished;
    /**
     * Terminates an ongoing smooth scroll
     */
    private _interrupted;
    /**
     * Deletes the destroyer function, runs if the smooth scroll has finished or interrupted
     */
    private _destroy;
    /**
     * A function called recursively that, given a context, steps through scrolling
     */
    private _step;
    private _applyScrollToOptions;
    /**
     * Scrolls to the specified offsets. This is a normalized version of the browser's native scrollTo
     * method, since browsers are not consistent about what scrollLeft means in RTL. For this method
     * left and right always refer to the left and right side of the scrolling container irrespective
     * of the layout direction. start and end refer to left and right in an LTR context and vice-versa
     * in an RTL context.
     * @param scrollable element
     * @param customOptions specified the offsets to scroll to.
     */
    scrollTo(scrollable: SmoothScrollElement, customOptions: SmoothScrollToOptions): Promise<void>;
    /**
     * Scroll to element by reference or selector
     */
    scrollToElement(scrollable: SmoothScrollElement, target: SmoothScrollElement, customOptions?: SmoothScrollToElementOptions): Promise<void>;
    static ɵfac: i0.ɵɵFactoryDeclaration<SmoothScrollManager, [null, null, { optional: true; }]>;
    static ɵprov: i0.ɵɵInjectableDeclaration<SmoothScrollManager>;
}
